@@ -3,26 +3,22 @@ package App::ModuleBuildTiny;
 use 5.008;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.003';
+our $VERSION = '0.004';
 
 use Exporter 5.57 'import';
 our @EXPORT = qw/modulebuildtiny/;
 
-use Archive::Tar;
 use Carp qw/croak/;
 use Config;
 use CPAN::Meta;
-use ExtUtils::Manifest qw/maniskip/;
+use ExtUtils::Manifest qw/manifind maniskip/;
 use File::Basename qw/basename dirname/;
 use File::Copy qw/copy/;
-use File::Find qw/find/;
 use File::Path qw/mkpath rmtree/;
-use File::Spec::Functions qw/catfile abs2rel rel2abs/;
+use File::Spec::Functions qw/catfile rel2abs/;
 use File::Temp qw/tempdir/;
 use Getopt::Long 2.39;
 use JSON::PP;
-use Module::CPANfile;
-use Module::Metadata;
 
 sub write_file {
 	my ($filename, $content) = @_;
@@ -39,30 +35,26 @@ sub prereqs_for {
 
 sub get_files {
 	my %opts = @_;
-	my %files;
-	my $skip = maniskip;
-	find({ 
-		no_chdir => 1,
-		wanted => sub {
-			my $name = abs2rel($_, '.');
-			$files{$name} = $name if -f $name and not $skip->($name);
-		},
-	}, '.');
+	my $maniskip = maniskip;
+	my $files = manifind();
+	delete $files->{$_} for @{ $opts{skip} }, grep { $maniskip->($_) } keys %$files;
 	
-	$files{'Build.PL'} ||= do {
+	$files->{'Build.PL'} ||= do {
 		my $minimum_mbt  = prereqs_for($opts{meta}, qw/configure requires Module::Build::Tiny/);
 		my $minimum_perl = prereqs_for($opts{meta}, qw/runtime requires perl 5.006/);
 		\"use $minimum_perl;\nuse Module::Build::Tiny $minimum_mbt;\nBuild_PL();\n";
 	};
-	$files{'META.json'} ||= \$opts{meta}->as_string;
-	$files{'META.yml'} ||= \$opts{meta}->as_string({ version => 1.4 });
-	$files{MANIFEST} ||= \join "\n", keys %files;
+	$files->{'META.json'} ||= \$opts{meta}->as_string;
+	$files->{'META.yml'} ||= \$opts{meta}->as_string({ version => 1.4 });
+	$files->{MANIFEST} ||= \join "\n", sort keys %$files;
 
-	return \%files;
+	return $files;
 }
 
 sub get_meta {
-	if (-e 'META.json' and -M 'META.json' < -M 'cpanfile') {
+	my %opts = @_;
+	my $mergefile = $opts{mergefile} || (grep { -f } qw/metamerge.json metamerge.yml/)[0];
+	if (not $opts{regenerate} and -e 'META.json' and -M 'META.json' < -M 'cpanfile' and (not $mergefile or -M 'META.json' < -M $mergefile)) {
 		return CPAN::Meta->load_file('META.json', { lazy_validation => 0 });
 	}
 	else {
@@ -70,15 +62,16 @@ sub get_meta {
 		$distname =~ s/(?:^(?:perl|p5)-|[\-\.]pm$)//x;
 		my $filename = catfile('lib', split /-/, $distname) . '.pm';
 
+		require Module::Metadata;
 		my $data = Module::Metadata->new_from_file($filename, collect_pod => 1);
 		my ($abstract) = $data->pod('NAME') =~ / \A \s+ \S+ \s? - \s? (.+?) \s* \z /x;
 		my $authors = [ map { / \A \s* (.+?) \s* \z /x } grep { /\S/ } split /\n/, $data->pod('AUTHOR') ];
 		my $version = $data->version($data->name)->stringify;
 
-		my $prereqs = -f 'cpanfile' ? Module::CPANfile->load('cpanfile')->prereq_specs : {};
+		my $prereqs = -f 'cpanfile' ? do { require Module::CPANfile; Module::CPANfile->load('cpanfile')->prereq_specs } : {};
 		$prereqs->{configure}{requires}{'Module::Build::Tiny'} ||= Module::Metadata->new_from_module('Module::Build::Tiny')->version->stringify;
 
-		my %metahash = (
+		my $metahash = {
 			name           => $distname,
 			version        => $version,
 			author         => $authors,
@@ -88,18 +81,31 @@ sub get_meta {
 			prereqs        => $prereqs,
 			release_status => $version =~ /_|-TRIAL$/ ? 'testing' : 'stable',
 			generated_by   => "App::ModuleBuildTiny version $VERSION",
-		);
-		return CPAN::Meta->create(\%metahash, { lazy_validation => 0 });
+			'meta-spec'    => {
+				version    => '2',
+				url        => 'http://search.cpan.org/perldoc?CPAN::Meta::Spec'
+			},
+		};
+		if ($mergefile && -r $mergefile) {
+			my $extra = Parse::CPAN::Meta->load_file($mergefile);
+			require CPAN::Meta::Merge;
+			$metahash = CPAN::Meta::Merge->new(default_version => '2')->merge($metahash, $extra);
+		}
+		return CPAN::Meta->create($metahash, { lazy_validation => 0 });
 	}
 }
 
+my @generatable = qw/Build.PL META.json META.yml MANIFEST/;
 my $parser = Getopt::Long::Parser->new(config => [qw/require_order pass_through gnu_compat/]);
 
 my %actions = (
 	dist => sub {
 		my %opts    = @_;
+		require Archive::Tar;
 		my $arch    = Archive::Tar->new;
-		my $content = get_files(%opts);
+		my $meta    = get_meta();
+		my $name    = $meta->name . '-' . $meta->version;
+		my $content = get_files(%opts, meta => $meta);
 		for my $filename (keys %{$content}) {
 			if (ref $content->{$filename}) {
 				$arch->add_data($filename, ${ $content->{$filename} });
@@ -109,15 +115,17 @@ my %actions = (
 			}
 		}
 		$_->mode($_->mode & ~oct 22) for $arch->get_files;
-		printf "tar czf $opts{name}.tar.gz %s\n", join ' ', keys %{$content} if ($opts{verbose} || 0) > 0;
-		$arch->write("$opts{name}.tar.gz", COMPRESS_GZIP, $opts{name});
+		printf "tar czf $name.tar.gz %s\n", join ' ', keys %{$content} if ($opts{verbose} || 0) > 0;
+		$arch->write("$name.tar.gz", &Archive::Tar::COMPRESS_GZIP, $name);
 	},
 	distdir => sub {
-		my %opts = @_;
-		mkpath($opts{name}, $opts{verbose}, oct '755');
-		my $content = get_files(%opts);
+		my %opts    = @_;
+		my $meta    = get_meta();
+		my $dir     = $opts{dir} || $meta->name . '-' . $meta->version;
+		mkpath($dir, $opts{verbose}, oct '755');
+		my $content = get_files(%opts, meta => $meta);
 		for my $filename (keys %{$content}) {
-			my $target = catfile($opts{name}, $filename);
+			my $target = catfile($dir, $filename);
 			mkpath(dirname($target)) if not -d dirname($target);
 			if (ref $content->{$filename}) {
 				write_file($target, ${ $content->{$filename} });
@@ -129,39 +137,42 @@ my %actions = (
 	},
 	test => sub {
 		my %opts = (@_, author => 1);
-		my $name = tempdir(CLEANUP => 1);
-		dispatch('distdir', %opts, name => $name);
+		my $dir  = tempdir(CLEANUP => 1);
+		dispatch('distdir', %opts, dir => $dir);
 		$parser->getoptionsfromarray($opts{arguments}, \%opts, qw/release! author!/);
 		my $env = join ' ', map { $opts{$_} ? uc($_) . '_TESTING=1' : () } qw/release author automated/;
-		system "cd '$name'; '$Config{perlpath}' Build.PL; ./Build; $env ./Build test";
+		system "cd '$dir'; '$Config{perlpath}' Build.PL; ./Build; $env ./Build test";
 	},
 	run => sub {
 		my %opts = @_;
-		my $name = tempdir(CLEANUP => 1);
-		dispatch('distdir', %opts, name => $name);
+		my $dir  = tempdir(CLEANUP => 1);
+		dispatch('distdir', %opts, dir => $dir);
 		my $command = @{ $opts{arguments} } ? join ' ', @{ $opts{arguments} } : $ENV{SHELL};
-		system "cd '$name'; '$Config{perlpath}' Build.PL; ./Build; $command";
+		system "cd '$dir'; '$Config{perlpath}' Build.PL; ./Build; $command";
 	},
 	listdeps => sub {
 		my %opts = @_;
 		$parser->getoptionsfromarray($opts{arguments}, \%opts, qw/json/);
+		my $meta = get_meta();
 		if (!$opts{json}) {
-			print "$_\n" for sort map { $opts{meta}->effective_prereqs->requirements_for($_, 'requires')->required_modules } qw/configure build test runtime/;
+			print "$_\n" for sort map { $meta->effective_prereqs->requirements_for($_, 'requires')->required_modules } qw/configure build test runtime/;
 		}
 		else {
-			my $hash = $opts{meta}->effective_prereqs->as_string_hash;
+			my $hash = $meta->effective_prereqs->as_string_hash;
 			print JSON::PP->new->ascii->pretty->encode($hash);
 		}
 	},
-	generate => sub {
+	regenerate => sub {
 		my %opts = @_;
-		my $content = get_files(%opts);
-		my @files = @{ $opts{arguments} };
+		my @files = @{ $opts{arguments} } ? @{ $opts{arguments} } : qw/Build.PL META.json META.yml MANIFEST/;
+
+		my $meta = get_meta(regenerate => scalar grep { /^META\./ } @files);
+		my $content = get_files(%opts, meta => $meta, skip => \@files);
 		for my $filename (@files) {
 			mkpath(dirname($filename)) if not -d dirname($filename);
 			write_file($filename, ${ $content->{$filename} }) if ref $content->{$filename};
 		}
-	}
+	},
 );
 
 sub dispatch {
@@ -175,8 +186,7 @@ sub modulebuildtiny {
 	my ($action, @arguments) = @_;
 	$parser->getoptionsfromarray(\@arguments, \my %opts, 'verbose!');
 	croak 'No action given' unless defined $action;
-	my $meta = get_meta();
-	return dispatch($action, %opts, arguments => \@arguments, meta => $meta, name => $meta->name . '-' . $meta->version);
+	return dispatch($action, %opts, arguments => \@arguments);
 }
 
 1;
@@ -187,7 +197,7 @@ App::ModuleBuildTiny - A standalone authoring tool for Module::Build::Tiny
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 DESCRIPTION
 
